/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
#include <stdio.h>
#include <pthread.h>
#include <string.h>
#include <config.h>

#include "file/interface.h"
#include "dxr2/interface.h"
#include "player.h"
#include "control.h"
#include "buffer.h"
#include "output.h"
#include "ifo_parse.h"
#include "read_config.h"
#include "cmdline-control.h"
#include "message.h"


int main ( int argc , char ** argv )
{
	player_info_t		* player_info;
	char				* err ;
	pthread_t 			control_thread ;
	pthread_t 			read_thread ;
	pthread_t 			write_thread ;
	pthread_attr_t		thread_attrib ;
	int					cur_opt ;
	output_plugin_fn	output ;

	player_info = malloc ( sizeof ( player_info_t ) ) ;
	memset ( player_info , 0 , sizeof ( player_info_t ) ) ;

	/* Load in information that cannot be changed
	* (firmware file, mountpoint, etc) and defaults for audio settings */
	err = read_config ( player_info , dxr2_config_file ) ;
	if ( err != NULL )
	{
		err = read_config ( player_info , alt_config_file ) ;
		if ( err != NULL )
		{
			printf ( "%s: Error reading config file %s: %s\n" ,
					ANNOUNCE , alt_config_file , err ) ;
			return 1 ;
		}
	}

	/* Parse command line options */
#ifdef HAVE_DXR2
	output = (output_plugin_fn) dxr2_plugin ;
#else
	output = (output_plugin_fn) file_plugin ;
#endif
	player_info -> immediate_playback = IP_OFF ;
	player_info -> immediate_playback_files = 0 ;
	player_info -> out_file = NULL ;

	while ( ( cur_opt = getopt ( argc , argv , "f:do" ) ) != -1 )
	{
		switch ( cur_opt )
		{
			case 'f' :		output = (output_plugin_fn) file_plugin ;
							if ( player_info -> out_file != NULL )
							{
								free ( player_info -> out_file ) ;
							}
							player_info -> out_file = 
									strdup ( optarg ) ;
							break ;
			case 'o' :		
#ifdef HAVE_DXR2
							output = (output_plugin_fn) dxr2_plugin_overlay ;
#endif
							break ;
			case 'd' :		player_info -> immediate_playback = IP_DVD ;
							break ;
			default :		printf ( "Usage:\n"
										"    %s [-f <file>] [-d] [files...]\n\n"
										"-f <file> selects file output mode, writing raw DVD data to <file>\n"
										"-d attempts to play a DVD immediately.\n"
#ifdef HAVE_DXR2
										"-o switches on dxr2 overlay mode\n"
#endif
										"\nIf any filenames are specified, then they will be played in sequence.\n" ,
									argv [ 0 ] ) ;
							return 1 ;
		}
	}
	if ( optind < argc )
	{
		player_info -> immediate_playback = IP_FILES ;
		for ( player_info -> immediate_playback_files = 0 ; ( optind < argc )
				&& ( player_info  -> immediate_playback_files < MAX_IP_FILES ) ;
				player_info  -> immediate_playback_files ++ )
		{
			player_info -> immediate_playback_names
				[ player_info  -> immediate_playback_files ] =
					strdup ( argv [ optind ] ) ;
			optind ++ ;
		}
	}

	/* Initialize thread stuff. */
	buf_setup ( player_info ) ;
	ifo_setup ( player_info ) ;
	msg_setup ( player_info ) ;

	err = NULL ;
	player_info -> out_plugin = 
			output_setup ( (void *) player_info , output , & err ) ;
	if ( err != NULL )
	{
		printf ( "%s: Error initialising output plugin: %s\n" ,
					ANNOUNCE , err ) ;
		return 1 ;
	}
	output_css_do_reset ( player_info -> out_plugin ,
					player_info -> dvd_device ) ;

	player_info -> view_mode = OUTPUT_VIEW_OFF ;

	/* Thread setup */
	pthread_attr_init(&thread_attrib);
    pthread_attr_setschedpolicy(&thread_attrib, SCHED_OTHER);
	pthread_create( &control_thread, &thread_attrib, cmdline_control_procedure , (void*) player_info);
	pthread_create( &read_thread, &thread_attrib, buf_reader , (void*) player_info);
	pthread_create( &write_thread, &thread_attrib, buf_writer , (void*) player_info);
 

	pthread_join(control_thread, NULL); 
	pthread_join(read_thread, NULL); 
	pthread_join(write_thread, NULL);  

	output_deinitialise ( player_info -> out_plugin ) ;

	return 0 ;
}

